#!/usr/bin/env python

'''py.test test cases to test OpenStudio.

These tests run based on webclient and need web2py server running.
'''

import datetime
import gluon.contrib.simplejson as sj
from gluon.contrib.populate import populate
from populate_os_tables import prepare_classes
from populate_os_tables import populate_classes
from populate_os_tables import populate_customers
from populate_os_tables import populate_customers_with_classcards
from populate_os_tables import populate_customers_with_subscriptions
from populate_os_tables import populate_school_classcards
from populate_os_tables import populate_workshop_activity_overlapping_class


def test_class_add(client, web2py):
    '''
        Can we add a class?
    '''
    populate(web2py.db.school_locations, 1)
    populate(web2py.db.school_classtypes, 1)

    url = '/classes/class_add'
    client.get(url)
    assert client.status == 200
    data = dict(school_locations_id='1',
                school_classtypes_id='1',
                Week_day='1',
                Starttime='06:00:00',
                Endtime='08:00:00',
                Startdate='2014-01-01',
                Maxstudents='123',
                MaxReservationsRecurring=100,
                MaxReservationsDT=23,
                )
    client.post(url, data=data)
    assert client.status == 200

    assert "Add teacher" in client.text # check redirection to add teacher page


    # move to schedule page and check display of input
    url = '/classes/schedule'
    client.get(url)
    assert client.status == 200

    assert web2py.db(web2py.db.classes).count() == 1
    assert data['Maxstudents'] in client.text
    query = web2py.db.classes.Maxstudents == data['Maxstudents']
    assert web2py.db(query).count() == 1


def test_class_edit(client, web2py):
    '''
        Can we edit a class?
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    url = '/classes/class_edit/?clsID=1'
    client.get(url)
    assert client.status == 200
    data = dict(id=1,
                school_locations_id='1',
                school_classtypes_id='1',
                Week_day='1', # Monday
                Starttime='06:00:00',
                Endtime='08:00:00',
                Startdate='2014-01-01',
                Maxstudents='499',
                MaxReservationsRecurring=400,
                MaxReservationsDT=99
                )
    client.post(url, data=data)
    assert client.status == 200
    assert "Schedule" in client.text # check redirection to schedule

    assert data['Maxstudents'] in client.text
    assert web2py.db(web2py.db.classes.Maxstudents == data['Maxstudents']).count() == 1


def test_schedule(client, web2py):
    '''
        Is the schedule showing all things as it should?
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    url = '/classes/schedule'
    client.get(url)
    assert client.status == 200
    assert 'Schedule' in client.text

    url = '/classes/schedule?year=2014&week=2' # we need the second week because the class is on a monday and the in week 1 monday is in december 2013.
    client.get(url)
    assert client.status == 200
    assert 'fa-user' in client.text
    assert 'Reservations' in client.text
    assert 'fa-pencil' in client.text
    assert 'fa-times' in client.text
    location_check = web2py.db.school_locations(1).Name.split(' ')[1]
    assert location_check in client.text

    # check if the label for subteacher is applied
    assert '<span class="os_label bg_light_blue">' in client.text


def test_schedule_classes_otc(client, web2py):
    '''
        Is a change from classes_otc showing in the schedule?
    '''
    populate_classes(web2py, with_otc=True)

    url = '/classes/schedule?year=2014&week=2'
    client.get(url)
    assert client.status == 200

    location = web2py.db.school_locations(2).Name.split(' ')[0]
    assert location in client.text

    classtype = web2py.db.school_classtypes(2).Name.split(' ')[0]
    assert classtype in client.text


def test_class_edit_on_date(client, web2py):
    '''
        Is the page to edit a single class working?
    '''
    prepare_classes(web2py)

    url = '/classes/class_edit_on_date?clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    data = {'Status':'open'}
    client.post(url, data=data)
    assert client.status == 200

    assert web2py.db(web2py.db.classes_otc).count() == 1

    client.get(url)
    assert client.status == 200
    assert '<option selected="selected" value="open">' in client.text


def test_class_edit_on_date_remove_changes(client, web2py):
    '''
        Can we remove the changes in classes_otc
    '''
    prepare_classes(web2py)

    web2py.db.classes_otc.insert(
        classes_id=1,
        ClassDate='2014-01-06',
        Status='cancelled' )

    web2py.db.commit()

    url = '/classes/class_edit_on_date_remove_changes?cotcID=1'
    client.get(url)
    assert client.status == 200
    assert web2py.db(web2py.db.classes_otc).count() == 0



def test_classes_otc_subteacher(client, web2py):
    '''
        Can we add substitute teachers for a class?
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    classes_id = '1'
    classdate = '2014-01-06'

    url = '/classes/class_edit_on_date?clsID=' + classes_id + '&date=' + classdate
    client.get(url)
    assert client.status == 200

    teachers_id = 2
    data = dict(auth_teacher_id=teachers_id,
                auth_teacher_id2='',
                teacher_role=1,
                teacher_role2='')
    client.post(url, data=data)
    assert client.status == 200

    query = (web2py.db.classes_otc.auth_teacher_id == teachers_id)
    assert web2py.db(query).count() == 1


def test_attendance_list_add_customer(client, web2py):
    '''
        Add a customer from the attendance list
    '''
    prepare_classes(web2py)

    rvars = 'clsID=1&date=2014-01-06'

    url = '/classes/attendance?' + rvars
    client.get(url)
    assert client.status == 200

    # now pretend we clicked the button and go to customers/add
    url = '/customers/add?' + rvars
    client.get(url)
    assert client.status == 200

    data = {'first_name' : "One Punch",
            'last_name'  : "Man",
            'email'      : 'Saitama_opm@yahoo.co.jp' }
    client.post(url, data=data)
    assert client.status == 200

    # verify redirection back to attendance_list
    assert 'Attendance' in client.text

    # verify DB
    query = (web2py.db.auth_user.first_name == data['first_name'])
    assert web2py.db(query).count() == 1


def test_expected_attendance_reservations(client, web2py):
    '''
        Does the expected attendance page show?
    '''
    populate_classes(web2py)
    populate_customers(web2py, 1)
    web2py.db.classes_reservation.insert(classes_id='1',
                                         auth_customer_id=1001,
                                         Startdate='2014-01-01')
    web2py.db.commit()

    assert web2py.db(web2py.db.auth_user).count() > 1
    assert web2py.db(web2py.db.classes).count() == 1
    assert web2py.db(web2py.db.classes_reservation).count() == 1

    url = '/classes/attendance?clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200
    assert web2py.db.auth_user(1001).first_name in client.text


def test_expected_attendance_from_previous_attendance_subscription(client, web2py):
    '''
        Does the expected attendance page show customers without reservations
        who attended using a class card or subscription in the past month?
    '''
    # get random url to init payment methods
    url = '/default/user/login'
    client.get(url)
    assert client.status == 200

    populate_classes(web2py)
    populate_customers_with_subscriptions(web2py)

    web2py.db.classes_attendance.insert(
        classes_id                 = 1,
        auth_customer_id           = 1001,
        customers_subscriptions_id = 1,
        ClassDate                  = '2014-01-06',
        AttendanceType             = None )

    web2py.db.commit()

    url = '/classes/attendance?clsID=1&date=2014-01-20'
    client.get(url)
    assert client.status == 200

    assert web2py.db.auth_user(1001).first_name in client.text


def test_expected_attendance_from_previous_attendance_classcard(client, web2py):
    '''
        Does the expected attendance page show customers without reservations
        who attended using a class card or subscription in the past month?
    '''
    # get random url to init payment methods
    url = '/default/user/login'
    client.get(url)
    assert client.status == 200

    populate_classes(web2py)
    populate_customers_with_classcards(web2py)

    web2py.db.classes_attendance.insert(
        classes_id                 = 1,
        auth_customer_id           = 1001,
        customers_classcards_id    = 1,
        ClassDate                  = '2014-01-06',
        AttendanceType             = 3 )

    web2py.db.commit()

    url = '/classes/attendance?clsID=1&date=2014-01-20'
    client.get(url)
    assert client.status == 200

    assert web2py.db.auth_user(1001).first_name in client.text


def test_attendance_sign_in_trialclass(client, web2py):
    '''
        Can we check in a customer for a trial class?
        Check using auth_user.id 1002
    '''
    classdate = '2014-01-06'
    prepare_classes(web2py)

    url = '/classes/attendance_sign_in_trialclass?cuID=1002&clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    # check we have count 2 for attendance in db
    query = (web2py.db.classes_attendance.id > 0)
    count = web2py.db(query).count()
    assert count == 4

    assert web2py.db(web2py.db.invoices).count() == 1
    invoice = web2py.db.invoices(1)
    assert invoice.invoices_groups_id == 100

    inv_clatt = web2py.db.invoices_classes_attendance(1)
    assert inv_clatt.classes_attendance_id == 5

    price = web2py.db.classes_price(1).Trial
    invoice_amount = web2py.db.invoices_amounts(1).TotalPriceVAT

    assert price == invoice_amount


def test_attendance_sign_in_dropin(client, web2py):
    '''
        Can we check in a customer for a drop in class?
        Check using auth_user.id 1002
    '''
    classdate = '2014-01-06'
    prepare_classes(web2py)

    url = '/classes/attendance_sign_in_dropin?cuID=1002&clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    # check we have count 2 for attendance in db
    query = (web2py.db.classes_attendance.id > 0)
    count = web2py.db(query).count()
    assert count == 4

    assert web2py.db(web2py.db.invoices).count() == 1
    invoice = web2py.db.invoices(1)
    assert invoice.invoices_groups_id == 100

    inv_clatt = web2py.db.invoices_classes_attendance(1)
    assert inv_clatt.classes_attendance_id == 5

    price = web2py.db.classes_price(1).Dropin
    invoice_amount = web2py.db.invoices_amounts(1).TotalPriceVAT

    assert price == invoice_amount


def test_attendance_sign_in_subscription(client, web2py):
    '''
        Can we check in a customer for a class using a subscription?
        Check using auth_user.id 1001
    '''
    # get random url to init payment methods for subscription
    client.get('/default/user/login')
    assert client.status == 200

    populate_customers_with_subscriptions(web2py)

    populate(web2py.db.school_locations, 2)
    populate(web2py.db.school_classtypes, 3)
    populate(web2py.db.school_levels, 3)
    web2py.db.classes.insert(school_locations_id=1,
                             school_classtypes_id=1,
                             Week_day=1,
                             Starttime='06:00:00',
                             Endtime='09:00:00',
                             Startdate='2014-01-01',
                             Enddate='2999-01-01',
                             Maxstudents=20,
                             )

    web2py.db.commit()

    url = '/classes/attendance_sign_in_subscription?cuID=1001&csID=1&clsID=1&date=2015-01-05'
    client.get(url)
    assert client.status == 200

    # check we have count 1 for attendance in db
    query = (web2py.db.classes_attendance.id > 0)
    count = web2py.db(query).count()
    assert count == 1


def test_attendance_sign_in_subscription_check_paused(client, web2py):
    '''
        Can we check in a customer for a class using a subscription
        does it show a message when the subscription is paused'?
        Check using auth_user.id 1001
    '''
    # get random url to init payment methods for subscription
    client.get('/default/user/login')
    assert client.status == 200

    populate_customers_with_subscriptions(web2py)

    populate(web2py.db.school_locations, 2)
    populate(web2py.db.school_classtypes, 3)
    populate(web2py.db.school_levels, 3)
    web2py.db.classes.insert(school_locations_id=1,
                             school_classtypes_id=1,
                             Week_day=1,
                             Starttime='06:00:00',
                             Endtime='09:00:00',
                             Startdate='2014-01-01',
                             Enddate='2999-01-01',
                             Maxstudents=20,
                             )

    web2py.db.customers_subscriptions_paused.insert(
        customers_subscriptions_id = 1,
        Startdate                  = '2014-01-01',
        Enddate                    = '2014-12-31',
        Description                = 'for testing')

    web2py.db.commit()

    url = '/classes/attendance_sign_in_subscription?cuID=1001&csID=1&clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    # check we have count 1 for attendance in db
    query = (web2py.db.classes_attendance.id > 0)
    count = web2py.db(query).count()
    assert count == 1
    assert 'Subscription is paused on this date' in client.text


def test_attendance_sign_in_subscription_weekly_classes_check(client, web2py):
    '''
        Can we check in a customer for a class using a subscription?
        Check using auth_user.id 1001
    '''
    # get random url to init payment methods for subscription
    client.get('/default/user/login')
    assert client.status == 200

    populate_customers_with_subscriptions(web2py)

    populate(web2py.db.school_locations, 2)
    populate(web2py.db.school_classtypes, 3)
    populate(web2py.db.school_levels, 3)
    web2py.db.classes.insert(school_locations_id=1,
                             school_classtypes_id=1,
                             Week_day=1,
                             Starttime='06:00:00',
                             Endtime='09:00:00',
                             Startdate='2014-01-01',
                             Enddate='2999-01-01',
                             Maxstudents=20,
                             )
    web2py.db.classes.insert(school_locations_id=1,
                             school_classtypes_id=1,
                             Week_day=2,
                             Starttime='06:00:00',
                             Endtime='09:00:00',
                             Startdate='2014-01-01',
                             Enddate='2999-01-01',
                             Maxstudents=20,
                             )

    web2py.db.commit()

    # Check in to a class
    url = '/classes/attendance_sign_in_subscription?cuID=1001&csID=1&clsID=1&date=2015-01-05'
    client.get(url)
    assert client.status == 200

    # Check in to another class
    url = '/classes/attendance_sign_in_subscription?cuID=1001&csID=1&clsID=2&date=2015-01-05'
    client.get(url)
    assert client.status == 200
    # now we should be over the limit
    assert 'exceeded' in client.text

    # check we have count 2 for attendance in db
    query = (web2py.db.classes_attendance.id > 0)
    count = web2py.db(query).count()
    assert count == 2


def test_attendance_sign_in_subscription_monthly_classes_check(client, web2py):
    '''
        Can we check in a customer for a class using a subscription?
        Check using auth_user.id 1001
    '''
    # get random url to init payment methods for subscription
    client.get('/default/user/login')
    assert client.status == 200

    populate_customers_with_subscriptions(web2py)

    # change first customer subscription to monthly one school_subscriptions_id = 3
    cs = web2py.db.customers_subscriptions(1)
    cs.school_subscriptions_id = 3
    cs.update_record()

    populate(web2py.db.school_locations, 2)
    populate(web2py.db.school_classtypes, 3)
    populate(web2py.db.school_levels, 3)
    web2py.db.classes.insert(school_locations_id=1,
                             school_classtypes_id=1,
                             Week_day=1,
                             Starttime='06:00:00',
                             Endtime='09:00:00',
                             Startdate='2014-01-01',
                             Enddate='2999-01-01',
                             Maxstudents=20,
                             )
    web2py.db.classes.insert(school_locations_id=1,
                             school_classtypes_id=1,
                             Week_day=2,
                             Starttime='06:00:00',
                             Endtime='09:00:00',
                             Startdate='2014-01-01',
                             Enddate='2999-01-01',
                             Maxstudents=20,
                             )

    web2py.db.commit()

    url = '/classes/attendance_sign_in_subscription?cuID=1001&csID=3&clsID=1&date=2015-01-05'
    client.get(url)
    assert client.status == 200

    url = '/classes/attendance_sign_in_subscription?cuID=1001&csID=3&clsID=2&date=2015-01-05'
    client.get(url)
    assert client.status == 200
    assert 'exceeded' in client.text

    # check we have count 2 for attendance in db
    query = (web2py.db.classes_attendance.id > 0)
    count = web2py.db(query).count()
    assert count == 2


def test_attendance_sign_in_classcard(client, web2py):
    '''
        Can we check in a customer for a class using a class card?
        Check using auth_user.id 1002
    '''
    classdate = '2014-01-06'
    populate_customers_with_classcards(web2py)

    populate(web2py.db.school_locations, 2)
    populate(web2py.db.school_classtypes, 3)
    populate(web2py.db.school_levels, 3)
    web2py.db.classes.insert(school_locations_id=1,
                             school_classtypes_id=1,
                             Week_day=1,
                             Starttime='06:00:00',
                             Endtime='09:00:00',
                             Startdate='2014-01-01',
                             Enddate='2999-01-01',
                             Maxstudents=20,
                             )
    web2py.db.commit()

    url = '/classes/attendance_sign_in_classcard?cuID=1002&ccdID=2&clsID=1&date=2015-01-05'
    client.get(url)
    assert client.status == 200

    # check we have count 2 for attendance in db
    query = (web2py.db.classes_attendance.id > 0)
    count = web2py.db(query).count()
    assert count == 1



def test_attendance_remove(client, web2py):
    '''
        Can we remove attendance using JSON after adding it?
    '''
    classdate = '2014-01-06'
    prepare_classes(web2py)
    assert web2py.db(web2py.db.classes_attendance).count() == 4

    # first visit the attendance page to set the session var used to determine where to redirect
    url = '/classes/attendance?clsID=1&date=' + classdate
    client.get(url)
    assert client.status == 200

    # now try to remove a customer from the attendance list
    url = '/classes/attendance_remove?clsID=1&cuID=1001&date=' + classdate
    client.get(url)
    assert client.status == 200

    # check redirect back to attendance
    assert 'Attendance' in client.text

    assert web2py.db(web2py.db.classes_attendance).count() == 2


def test_attendance_classcard_class_count_classestaken(client, web2py):
    '''
        Is the class count increased when a class is added to a customer?
    '''
    populate_classes(web2py)

    nr_cards = 1
    populate_school_classcards(web2py, nr_cards, trialcard = True)
    populate_customers(web2py, 1)

    web2py.db.customers_classcards.insert(
        school_classcards_id = 1,
        auth_customer_id     = 1001,
        Startdate            = '2014-01-01')
    web2py.db.commit()

    # test adding class and check count
    url = '/classes/attendance_sign_in_classcard?cuID=1001&clsID=1&ccdID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    assert web2py.db(web2py.db.classes_attendance).count() == 1
    assert web2py.db.classes_attendance(1).customers_classcards_id == 1

    classcard = web2py.db.customers_classcards(1)
    assert classcard.ClassesTaken == 1

    # test count after removing
    url = '/classes/attendance_remove?cuID=1001&clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    classcard = web2py.db.customers_classcards(1)
    assert classcard.ClassesTaken == 0


def test_classes_override_attendance(client, web2py):
    '''
        Does the override attendance page work?
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    url = '/classes/attendance_override?clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    data = dict(Amount='123456')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Saved' in client.text # check redirection to same page
    query = (web2py.db.classes_attendance_override.Amount == \
             data['Amount'])
    assert web2py.db(query).count() == 1

    # now can we edit it

    client.get(url)
    assert client.status == 200
    assert data['Amount'] in client.text

    update_data = dict(id=1,
                       Amount='987654')
    client.post(url, data=update_data)
    assert client.status == 200
    assert 'Saved' in client.text # check redirection to same page
    query = (web2py.db.classes_attendance_override.Amount == \
             update_data['Amount'])
    assert web2py.db(query).count() == 1


def test_classes_otc_status_cancelled(client, web2py):
    '''
        Does the cancel status work?
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    classes_id = '1'
    classdate = '2014-01-06'

    # check cancelling class
    url = '/classes/class_edit_on_date?clsID=' + classes_id + '&date=' + classdate
    client.get(url)
    assert client.status == 200

    data = {
        'Status':'cancelled'
    }

    client.post(url, data=data)
    assert client.status == 200
    assert 'Saved' in client.text

    cotc = web2py.db.classes_otc(1)
    assert cotc.Status == data['Status']


def test_classes_otc_status_open(client, web2py):
    '''
        Does the cancel status work?
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    classes_id = '1'
    classdate = '2014-01-06'

    # check cancelling class
    url = '/classes/class_edit_on_date?clsID=' + classes_id + '&date=' + classdate
    client.get(url)
    assert client.status == 200

    data = {
        'Status':'open'
    }

    client.post(url, data=data)
    assert client.status == 200
    assert 'Saved' in client.text

    cotc = web2py.db.classes_otc(1)
    assert cotc.Status == data['Status']



def test_classes_open(client, web2py):
    '''
        Check if classes appear on the classes_open list
    '''
    def next_weekday(d, weekday):
        days_ahead = weekday - d.weekday()
        if days_ahead <= 0: # Target day already happened this week
            days_ahead += 7
        return d + datetime.timedelta(days_ahead)

    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    # get 2 mondays later than today
    delta = datetime.timedelta(days=7)
    today = datetime.date.today()
    next_monday = next_weekday(today, 0) # 0 = Monday, 1=Tuesday, 2=Wednesday...
    monday_after_that = next_monday + delta

    web2py.db.classes_otc.insert(
        Status = 'open',
        classes_id = 1,
        ClassDate = next_monday )
    web2py.db.classes_otc.insert(
        Status = 'open',
        classes_id = 1,
        ClassDate = monday_after_that,
        school_locations_id = 2,
        school_classtypes_id = 2)
    web2py.db.classes_otc.insert(
        Status = 'open',
        classes_id = 1,
        ClassDate = '2014-01-06' )

    web2py.db.commit()

    assert web2py.db(web2py.db.classes_otc.Status == 'open').count() == 3

    # get the page
    url = '/classes/classes_open'
    client.get(url)
    assert client.status == 200

    # past open classes shouldn't be shown
    assert '2014-01-06' not in client.text
    # future ones should be shown
    assert unicode(next_monday) in client.text
    assert unicode(monday_after_that) in client.text

    # check location & classtype from classes_otc
    location = web2py.db.school_locations(2).Name.split(' ')[0]
    assert location in client.text
    classtype = web2py.db.school_classtypes(2).Name.split(' ')[0]
    assert classtype in client.text


def test_overlapping_workshops(client, web2py):
    '''
        Test if the overlapping workshop activities show up
    '''
    populate_workshop_activity_overlapping_class(web2py)

    # set schedule session vars first
    url = '/classes/schedule?schedule_year=2014&schedule_week=2'
    client.get(url)
    assert client.status == 200
    assert '2014-01-06' in client.text # make sure we shifted weeks

    # test overlapping page
    url = '/classes/overlapping_workshops'
    client.get(url)
    assert client.status == 200
    activity = web2py.db.workshops_activities(1)
    assert activity.Activity.split(' ')[0] in client.text


def test_overlapping_workshops_count(client, web2py):
    '''
        Test if the count shows up correctly for overlapping workshops
        in the shedule
    '''
    populate_workshop_activity_overlapping_class(web2py)

    # set schedule session vars first
    url = '/classes/schedule?schedule_year=2014&schedule_week=2'
    client.get(url)
    assert client.status == 200

    url = '/classes/schedule_get_overlapping_workshops.load'
    client.get(url)
    assert client.status == 200

    # check badge on the schedule page
    assert '<span class="badge">1</span>' in client.text


def test_class_teachers(client, web2py):
    '''
        Test if we can get a list of teachers for a class
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    url = '/classes/class_teachers?clsID=1'
    client.get(url)
    assert client.status == 200

    teacher = web2py.db.auth_user(2)

    assert teacher.first_name in client.text


def test_class_teacher_add(client, web2py):
    '''
        Test if we can add a teacher
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    url = '/classes/class_teacher_add?clsID=1'
    client.get(url)
    assert client.status == 200

    data = dict(auth_teacher_id=2,
                Startdate='2014-01-01',
                Enddate='2014-02-01')
    client.post(url, data=data)
    assert client.status == 200

    assert web2py.db(web2py.db.classes_teachers).count() == 2
    teacher = web2py.db.auth_user(2)

    assert teacher.first_name in client.text

def test_class_teacher_edit(client, web2py):
    '''
        Test if we can edit a teacher
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    url = '/classes/class_teacher_edit?clsID=1&cltID=1'
    client.get(url)
    assert client.status == 200

    data = dict(id=1,
                classes_id=1,
                auth_teacher_id=3,
                Startdate='2014-01-01',
                Enddate='2014-02-01')
    client.post(url, data=data)
    assert client.status == 200

    teacher = web2py.db.auth_user(3)
    assert teacher.first_name in client.text


def test_class_prices(client, web2py):
    '''
        Test if we can get a list of prices for a class
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    url = '/classes/class_teachers?clsID=1'
    client.get(url)
    assert client.status == 200

    class_price = web2py.db.classes_price(1)

    assert unicode(class_price.Startdate) in client.text


def test_class_price_add(client, web2py):
    '''
        Test if we can add a price
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    url = '/classes/class_price_add?clsID=1'
    client.get(url)
    assert client.status == 200

    data = dict(Dropin=254098303,
                tax_rates_id_dropin=1,
                Trial=1324243,
                tax_rates_id_trial=1,
                Startdate='2014-01-01',
                Enddate='2014-02-01')
    client.post(url, data=data)
    assert client.status == 200

    assert web2py.db(web2py.db.classes_price).count() == 2
    price = format(web2py.db.classes_price(2).Dropin, '.2f')

    assert price in client.text


def test_class_price_edit(client, web2py):
    '''
        Test if we can edit a teacher
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    url = '/classes/class_price_edit?clsID=1&clpID=1'
    client.get(url)
    assert client.status == 200

    data = dict(id=1,
                Dropin=254098303,
                tax_rates_id_dropin=1,
                Trial=1324243,
                tax_rates_id_trial=1,
                Startdate='2014-01-01',
                Enddate='2014-02-01')
    client.post(url, data=data)
    assert client.status == 200

    assert web2py.db(web2py.db.classes_price).count() == 1

    price = format(web2py.db.classes_price(1).Dropin, '.2f')

    assert price in client.text


def test_class_teacher_display_schedule(client, web2py):
    '''
        Test whether a teacher shows up currectly in the schedule
    '''
    populate_classes(web2py)
    assert web2py.db(web2py.db.classes).count() == 1

    url = '/classes/schedule'
    client.get(url)
    assert client.status == 200

    teacher = web2py.db.auth_user(2)
    assert teacher.first_name in client.text


#TODO: move this test to reports
# def test_teacher_classes(client, web2py):
#     '''
#         Test if classes are listed for a teacher
#     '''
#     populate_classes(web2py)
#
#     url = '/classes/teacher_classes?teachers_id=2&year=2014&month=1'
#     client.get(url)
#     assert client.status == 200
#
#     location = web2py.db.school_locations(1).Name.split(' ')[0]
#     assert location in client.text

#TODO: move this test to reports
# def test_teacher_classes_role_display(client, web2py):
#     '''
#         Test if the role for teacher 2 is displayed correctly
#     '''
#     populate_classes(web2py)
#
#     url = '/classes/teacher_classes?teachers_id=2&year=2014&month=1'
#     client.get(url)
#     assert client.status == 200
#
#     # check if the label for subteacher is applied
#     assert '<span class="os_label bg_light_blue">' in client.text


def test_school_holiday_display_schedule(client, web2py):
    '''
        Is a holiday displayed properly in the schedule?
    '''
    populate_classes(web2py)
    description = 'fslkdfjlkrjlkdf'
    web2py.db.school_holidays.insert(Description=description,
                                     Startdate='2014-01-01',
                                     Enddate='2014-01-30',
                                     Classes=True)
    web2py.db.school_holidays_locations.insert(
        school_locations_id = 1,
        school_holidays_id = 1)
    web2py.db.commit()

    url = '/classes/schedule?year=2014&week=2'
    client.get(url)
    assert client.status == 200

    assert description in client.text


def test_reservations(client, web2py):
    '''
        Test list of reservations
    '''
    prepare_classes(web2py)

    url = '/classes/reservations?filter=this&clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    assert client.text.count('Weekly from 2014-01-01') == 1
    assert client.text.count('Drop in class on 2014-01-06') == 1
    assert client.text.count('Trial class on 2014-01-06') == 1

def test_resevations_maxstudents_reached(client, web2py):
    '''
        Does a warning message show on the reservations page when the
        reservations count exceeds the Maxstudents for a class?
    '''
    prepare_classes(web2py)

    # lower number of available spaces to get a warning
    cls = web2py.db.classes(1)
    cls.Maxstudents = 1
    cls.update_record()

    web2py.db.commit()

    url = '/classes/reservations?clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    assert 'Warning' in client.text


def test_reservation_maxreservations_recurring_reached(client, web2py):
    '''
        Does a warning message show on the reservations page when the
        reservations count exceeds the Maxstudents for a class?
    '''
    prepare_classes(web2py)

    # lower number of available spaces to get a warning
    cls = web2py.db.classes(1)
    cls.MaxReservationsRecurring = 1
    cls.update_record()

    # another recurring class reservation
    web2py.db.classes_reservation.insert(auth_customer_id=1002,
                                         classes_id='1',
                                         Startdate='2014-01-01',
                                         SingleClass=False,
                                         TrialClass=False)

    web2py.db.commit()

    url = '/classes/reservations?clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    assert 'Warning' in client.text


# def test_reservation_maxreservations_dropin_trial_reached(client, web2py):
#     '''
#         Does a warning message show on the reservations page when the
#         reservations count exceeds the Maxstudents for a class?
#     '''
#     prepare_classes(web2py)
#
#     # lower number of available spaces to get a warning
#     cls = web2py.db.classes(1)
#     cls.MaxReservationsDT = 1
#     cls.update_record()
#
#     # another dropin class reservation
#     web2py.db.classes_reservation.insert(auth_customer_id=1002,
#                                          classes_id='1',
#                                          Startdate='2014-01-06',
#                                          SingleClass=True,
#                                          TrialClass=False)
#
#     web2py.db.commit()
#
#     url = '/classes/reservations?clsID=1&date=2014-01-06'
#     client.get(url)
#     assert client.status == 200
#
#     assert 'Warning' in client.text

def test_reservations_recurring(client, web2py):
    '''
        List reservations for a customer (this is the default)
    '''
    prepare_classes(web2py)

    url = '/classes/reservations?filter=recurring&clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    assert client.text.count('Weekly from 2014-01-01') == 1


def test_reservations_filter_single(client, web2py):
    '''
        Check filter for single classes
    '''
    prepare_classes(web2py)

    url = '/classes/reservations?filter=single&clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    assert client.text.count('Drop in class on 2014-01-06') == 1


def test_reservations_filter_trial(client, web2py):
    '''
        Check filter for trialclasses
    '''
    prepare_classes(web2py)

    url = '/classes/reservations?filter=trial&clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    assert client.text.count('Trial class on 2014-01-06') == 1


def test_reservation_cancel(client, web2py):
    '''
        Test cancelling of a reservation
    '''
    prepare_classes(web2py)

    # get url to set session variable to return back to reservations
    res_url = '/classes/attendance?clsID=1&date=2014-01-06'
    client.get(res_url)
    assert client.status == 200

    # cancel reservation and check
    url = '/classes/reservation_cancel?crID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    # check db
    assert web2py.db(web2py.db.classes_reservation_cancelled).count() == 1

    # check uncancel
    url = '/classes/reservation_cancel?crID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    # check db
    assert web2py.db(web2py.db.classes_reservation_cancelled).count() == 0



def test_notes(client, web2py):
    '''
        Test list of class notes
    '''
    prepare_classes(web2py)

    note = web2py.db.classes_notes(1)

    url = '/classes/notes?clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    assert note['Note'] in client.text


def test_notes_add(client, web2py):
    '''
        Can we add a note?
    '''
    prepare_classes(web2py)

    url = '/classes/notes?clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200

    data = {'classes_id' : 1,
            'ClassDate' : '2014-01-06',
            'auth_user' : 1002,
            'TeacherNote' : True,
            'Note' : 'Cherries'}

    client.post(url, data=data)
    assert client.status == 200

    assert data['Note'] in client.text
    assert web2py.db(web2py.db.classes_notes).count() == 2


def test_notes_edit(client, web2py):
    '''
        Can we edit a note?
    '''
    prepare_classes(web2py)

    url = '/classes/note_edit?clsID=1&date=2014-01-06&cnID=1'
    client.get(url)
    assert client.status == 200

    data = {'id' : 1,
            'classes_id' : 1,
            'ClassDate' : '2014-01-06',
            'auth_user' : 1002,
            'TeacherNote' : True,
            'Note' : 'Cherries'}

    client.post(url, data=data)
    assert client.status == 200

    assert data['Note'] in client.text
    assert web2py.db(web2py.db.classes_notes).count() == 1


def test_notes_delete(client, web2py):
    '''
        Can we delete a note?
    '''
    prepare_classes(web2py)

    url = '/classes/note_delete?clsID=1&date=2014-01-06&cnID=1'
    client.get(url)
    assert client.status == 200

    assert web2py.db(web2py.db.classes_notes).count() == 0
